//______________________________________________________________________________
//
//  Main form of threaded message demo
//  Written for an article in Blaise Pascal Magazine
//
//  Feel free to alter and experiment with it.
//  Also feel free to ask questions or contribute or comment to this ideas.
//
//  J.P.Hoefnagel, 1M2M B.V.
//  EMail: jph@1m2m.eu
//______________________________________________________________________________
//

unit fBPM_MessageDemo;

interface

uses
  Vcl.StdCtrls, Vcl.ComCtrls, Vcl.Controls, Vcl.Buttons, Vcl.Forms,Vcl.AppEvnts,
  Vcl.ExtCtrls, Vcl.Graphics,
  System.Classes,System.UITypes,system.SysUtils,System.Types,
  uBPM_BaseThread,
  uBPM_Messages,
  uBPM_task1;

type
  TfrmBPM_MessageDemo = class(TForm)
    edLog     : TRichEdit;
    btnActive : TSpeedButton;
    SBar      : TStatusBar;
    edText    : TEdit;
    cbShowMaint1000: TCheckBox;
    edTime    : TEdit;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnActiveClick(Sender: TObject);
    procedure edTextChange(Sender: TObject);
    procedure cbShowMaint1000Click(Sender: TObject);
  private
    BlockOnChangeEvents:boolean;
    MsgID_GUI : TBPM_MessageID;
    MsgID_Task1: TBPM_MessageID;
    MsgHan    : THandle;
    AppEvt    : TApplicationEvents;
    SecTimer  : TTimer;
    InitReq   : boolean;
    procedure OnInit();
    procedure OnDone();
    procedure OnMaint(var Done: Boolean);
    procedure OnMaint1000();
    procedure OnMsgHandler(MR:TBPM_MessageRec);
    procedure log(Cmd: integer; Msg: string; Color: TColor);
    procedure OnIdle(Sender: TObject; var Done: Boolean);
    procedure OnSecTimer(Sender:TObject);
  public
  end;

var
  frmBPM_MessageDemo: TfrmBPM_MessageDemo;

implementation

{$R *.dfm}

// library function
procedure ColLog(LB:TRichEdit;s:string;col:tcolor;DoUpdate:boolean=false;limitlines:boolean=true);
var h:integer;
begin
  for h:=1 to length(s) do if s[h]<' ' then s[h]:='|';
  LB.CaretPos:=point(0,lb.lines.count);
  LB.SelAttributes.Color:=col;  LB.Lines.add(s);
  h:=LB.ClientHeight; if h<26 then h:=26;
  lb.Font.Height;
  if limitlines then while LB.lines.count>=h div 14 do LB.Lines.Delete(0);
  if DoUpdate then lb.Update;
end;

// library function
procedure SetSpdButtonLed(btn:TSpeedButton; color:TColor);
var  bm: TBitmap;sz,xo,yo:integer;
begin
  sz:=13; xo:=0; yo:=0;
  bm:=tbitmap.create(); bm.Transparent:=true;
  bm.Width  := sz-1; bm.Height := sz-1;
  with bm.Canvas do begin
    pen.color:=clWhite;  moveto(xo,yo+sz-3);lineto(xo     ,yo     );lineto(xo+sz-1,yo);
    pen.color:=clDkGray; moveto(xo,yo+sz-2);lineto(xo+sz-2,yo+sz-2);lineto(xo+sz-2,yo);
    brush.color:=color;
    pen.color:=clLtGray; rectangle(xo+1,yo+1,xo+sz-2,yo+sz-2);
  end;
  btn.Glyph.Assign(bm);
  bm.free;
end;

//------------------------------------------------------------------------------
// Form code to make calls to generic functions OnMaint(), OnMsgHandler(),
//   OnMaint1000(), OnInit(), OnDone() which are also defined in BPM Thread objects
//------------------------------------------------------------------------------

procedure TfrmBPM_MessageDemo.FormCreate(Sender: TObject);
begin
  InitReq           := true;
  SecTimer          := TTimer.Create(self);
  SecTimer.enabled  := False;
  SecTimer.interval := 1000;
  SecTimer.OnTimer  := OnSecTimer;
  AppEvt            := TApplicationEvents.Create(self);
  AppEvt.OnIdle     := OnIdle;
  MsgID_GUI         := BPM_RegisterMsgID('BPM_MainLogWindow');
  MsgID_Task1       := BPM_RegisterMsgID('BPM_Task1');
  MsgHan            := BPM_MsgBufOpen(-1); // listen to all messages, not only MsgID
end;

procedure TfrmBPM_MessageDemo.FormShow(Sender: TObject);
begin
  if InitReq then begin
    InitReq := false;
    OnInit();
    SecTimer.enabled := True;
  end;
end;

procedure TfrmBPM_MessageDemo.OnSecTimer(Sender:TObject);
begin
  OnMaint1000();
end;

procedure TfrmBPM_MessageDemo.OnIdle(Sender: TObject; var Done: Boolean);
var MR:TBPM_MessageRec;
begin
  OnMaint(Done);
  while (BPM_MsgBufRead(MsgHan,MR)) do begin
    Done := false;
    OnMsgHandler(MR);
  end;
end;

procedure TfrmBPM_MessageDemo.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  EndAllThreadsRequest := True;
  SecTimer.enabled     := False;
  OnDone();
end;

//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------

procedure TfrmBPM_MessageDemo.log(Cmd:integer; Msg:string; Color:TColor);
begin
  BPM_MsgBufWrite(MsgID_GUI,Cmd,Msg,Color);
end;

procedure TfrmBPM_MessageDemo.OnInit();
begin
  BlockOnChangeEvents:=false;
  Task1.active:=btnActive.Down;
end;

procedure TfrmBPM_MessageDemo.OnDone();
begin
//
end;

procedure TfrmBPM_MessageDemo.OnMaint(var Done: Boolean);
begin
//
end;

procedure TfrmBPM_MessageDemo.OnMaint1000();
begin
  if cbShowMaint1000.Checked then
    log(0,'Form.OnMaint1000()',clMaroon);
  SBar.Panels[0].Text:=format('Task1 CPU %4.2f %%',[Task1.GetCPULoad()]);
end;

procedure TfrmBPM_MessageDemo.OnMsgHandler(MR: TBPM_MessageRec);
begin
  // command codes returned by BPMRegisterCmd are $8000 or higher
  if (MR.Cmd>=$8000) then begin
    BlockOnChangeEvents:=true;
    try
      if (MR.Cmd = CmdBtnActiveLed)     then SetSpdButtonLed(btnActive,mr.color);
      if (MR.Cmd = task1.cmd_UpdEDTime) then edTime.Text:=mr.Msg;
    finally
      BlockOnChangeEvents:=false;
    end;
  end else begin // Command codes lower than 0x8000 are shown as colored text
    collog(edLog,format('%4.4x:%4.4x %s',[MR.MsgID,MR.Cmd,MR.Msg]),MR.Color);
  end;
end;

procedure TfrmBPM_MessageDemo.btnActiveClick(Sender: TObject);
begin
  if BlockOnChangeEvents then exit;
  Task1.active:=btnActive.Down;
end;

procedure TfrmBPM_MessageDemo.edTextChange(Sender: TObject);
begin
  if BlockOnChangeEvents then exit;
  BPM_MsgBufWrite(MsgID_Task1,Task1.cmd_OnEDTextChange,edText.Text,0);
end;

procedure TfrmBPM_MessageDemo.cbShowMaint1000Click(Sender: TObject);
var msg:string;
begin
  if BlockOnChangeEvents then exit;
  if cbShowMaint1000.Checked then msg:='1' else msg:='0';
  BPM_MsgBufWrite(MsgID_Task1,Task1.cmd_OnCBMaint1000Click,msg,0)
end;

end.
