//______________________________________________________________________________
//
//  Example of a messsage-aware thread object.
//  Do not expect useful functionality, it is for demonstration only.
//  Written for an article in Blaise Pascal Magazine
//
//  Feel free to modify and experiment with it.
//  Also feel free to ask questions or contribute or comment to this ideas.
//
//  J.P.Hoefnagel, 1M2M B.V.
//  EMail: jph@1m2m.eu
//______________________________________________________________________________
//
unit uBPM_Task1;

interface uses
  System.Classes,System.SysUtils,system.UITypes,Graphics,
  uBPM_Messages,
  uBPM_BaseThread;

type
  TTask1 = class(TBPM_BaseThread)
  private
    ShowMaint1000:boolean;
    LogMsgID : TBPM_MessageID;
    procedure log(Cmd: integer; Msg: string; Color: TColor);
  public
    cmd_OnEDTextChange     :TBPM_CommandID;
    cmd_OnCBMaint1000Click :TBPM_CommandID;
    cmd_UpdEDTime          :TBPM_CommandID;
    constructor Create();
    procedure   OnMaint(var Done: Boolean);       override;
    procedure   OnMaint1000();                    override;
    procedure   OnBeforeExecute();                override;
    procedure   OnAfterExecute();                 override;
    procedure   OnActivate();                     override;
    procedure   OnDeActivate();                   override;
    procedure   OnMsgHandler(mr:TBPM_MessageRec); override;
    procedure   OnErrorlog(Cde:integer;Msg:string;Color:TColor); override;
  end;

function Task1():TTask1;

implementation

//Singleton Task1
var fTask1:TTask1;
function Task1():TTask1;
begin
  if not assigned(fTask1) then fTask1:=TTask1.Create();
  result:=fTask1;
end;

{ TTask1 }

constructor TTask1.Create;
begin
  inherited;
  ShowMaint1000         :=false;
  LogMsgID              :=BPM_RegisterMsgID('BPM_MainLogWindow');
  MsgID                 :=BPM_RegisterMsgID('BPM_Task1');
  cmd_OnCBMaint1000Click:=BPM_RegisterCmdID('OnCBMaint1000Click');
  cmd_OnEDTextChange    :=BPM_RegisterCmdID('OnEDTextChange');
  cmd_UpdEDTime         :=BPM_RegisterCmdID('UpdEDTime');
end;

procedure TTask1.log(Cmd:integer; Msg:string; Color:TColor);
begin
  BPM_MsgBufWrite(LogMsgID,Cmd,Msg,Color);
end;

procedure TTask1.OnActivate;
begin
  log(CmdBtnActiveLed,'',clLime);
  log(0,'Task1.OnActivate();',clGreen);
end;

procedure TTask1.OnAfterExecute;
begin
  log(0,'Task1.OnAfterExecute();',clBlue);
end;

procedure TTask1.OnBeforeExecute;
begin
  log(0,'Task1.OnBeforeExecute();',clBlue);
end;

procedure TTask1.OnDeActivate;
begin
  log(CmdBtnActiveLed,'',clRed);
  log(0,'Task1.OnDeactivate();',clRed);
end;

procedure TTask1.OnErrorlog(Cde: integer; Msg: string; Color: TColor);
begin
  log(Cde,'Task1.OnErrorlog('+msg+');',Color);
end;

procedure TTask1.OnMaint;
begin
end;

procedure TTask1.OnMaint1000;
begin
  if ShowMaint1000 then log(0,'Task1.OnMaint1000();',clBlack);
  log(cmd_UpdEDTime,timetostr(now),0);
end;

procedure TTask1.OnMsgHandler(mr: TBPM_MessageRec);
begin
  if mr.Cmd = cmd_OnEDTextChange then begin
    log(0,'Edit1 updated to '+mr.Msg,clFuchsia);
  end else
  if mr.Cmd = cmd_OnCBMaint1000Click then begin
    ShowMaint1000 := (mr.Msg='1');
  end;
end;

initialization
  fTask1:=nil;
end.
